@@ -30,6 +30,15 @@ class NoIntersectionError(RuntimeError):
         super().__init__(message)
         self.point = point
 
+class TabError(RuntimeError):
+    def __init__(self, origin, direction, hints):
+        self.origin = origin
+        self.direction = direction
+        message = "Cannot create tab; possible causes:\n"
+        for hint in hints:
+            message += f"- {hint}\n"
+        super().__init__(message)
+
 class TabFilletError(RuntimeError):
     pass
 
@@ -93,7 +102,7 @@ def isValidPcbShape(g):
     with zero length. Unfortunately, KiCAD does not discard such lines when
     saving. Therefore, we have to check it.
     """
-    return g.GetShape() != pcbnew.S_SEGMENT or g.GetLength() > 0
+    return g.GetShape() != pcbnew.S_SEGMENT or g.GetLength() >= fromMm(0.001)
 
 def extractRings(geometryList):
     """
@@ -201,6 +210,8 @@ def approximateBezier(bezier, endWith):
                      3 * t ** 2 * (1 - t) * bc2 + \
                      t ** 3 * end
             outline.append(vertex)
+    else:
+        outline += [start, end]
     outline.append(end)
 
     endWith = np.array(endWith)
@@ -694,7 +705,7 @@ class Substrate:
                     if not candidateCircle.addPoint(coords[j]):
                         break
 
-            if candidateCircle.foundCircle is not None:
+            if candidateCircle.foundCircle is not None and candidateCircle.foundCircle[1] > fromMm(0.25):
                 center, radius = candidateCircle.foundCircle
                 start, end, mid = candidateCircle.start, candidateCircle.end, candidateCircle.mid
 
@@ -822,10 +833,14 @@ class Substrate:
         """
         self.orient()
 
-        origin = np.array(origin)
+        if self.substrates.contains(Point(origin)) and not self.substrates.boundary.contains(Point(origin)):
+            raise TabError(origin, direction, ["Tab annotation is placed inside the board. It has to be on edge or outside the board."])
+
+        origin = np.array(origin, dtype=np.float64)
+        direction = np.around(normalize(direction), 4)
+        origin -= direction * float(SHP_EPSILON)
         for geom in listGeometries(self.substrates):
             try:
-                direction = np.around(normalize(direction), 4)
                 sideOriginA = origin + makePerpendicular(direction) * width / 2
                 sideOriginB = origin - makePerpendicular(direction) * width / 2
                 boundary = geom.exterior
@@ -844,9 +859,9 @@ class Substrate:
                 direction = -direction
                 for p in listGeometries(partitionLine):
                     try:
-                        partitionSplitPointA = closestIntersectionPoint(splitPointA.coords[0],
+                        partitionSplitPointA = closestIntersectionPoint(splitPointA.coords[0] - direction * float(SHP_EPSILON),
                                 direction, p, maxHeight)
-                        partitionSplitPointB = closestIntersectionPoint(splitPointB.coords[0],
+                        partitionSplitPointB = closestIntersectionPoint(splitPointB.coords[0] - direction * float(SHP_EPSILON),
                                 direction, p, maxHeight)
                     except NoIntersectionError: # We cannot span towards the partition line
                         continue
@@ -866,27 +881,21 @@ class Substrate:
                         # penetrate the board substrate. Otherwise, there is a
                         # numerical instability on small slopes that yields
                         # artifacts on substrate union
-                        offsetTabFace = [(p[0] - SHP_EPSILON * direction[0], p[1] - SHP_EPSILON * direction[1]) for p in tabFace.coords]
-                        partitionFaceCoord = [(p[0] + SHP_EPSILON * direction[0], p[1] + SHP_EPSILON * direction[1]) for p in partitionFaceCoord]
+                        offsetTabFace = [(p[0] - float(SHP_EPSILON) * direction[0], p[1] - float(SHP_EPSILON) * direction[1]) for p in tabFace.coords]
+                        partitionFaceCoord = [(p[0] + float(SHP_EPSILON) * direction[0], p[1] + float(SHP_EPSILON) * direction[1]) for p in partitionFaceCoord]
                         tab = Polygon(offsetTabFace + partitionFaceCoord)
                         return self._makeTabFillet(tab, tabFace, fillet)
                 return None, None
             except NoIntersectionError as e:
                 continue
             except TabFilletError as e:
-                message = f"Cannot create fillet for tab: {e}\n"
-                message += f"  Annotation position {self._strPosition(origin)}\n"
-                message += "This is a bug. Please open an issue and provide the board on which the fillet failed."
-                raise RuntimeError(message) from None
+                raise TabError(origin, direction, ["This is a bug. Please open an issue and provide the board on which the fillet failed."])
 
-        message = "Cannot create tab:\n"
-        message += f"  Annotation position {self._strPosition(origin)}\n"
-        message += f"  Tab ray origin that failed: {self._strPosition(origin)}\n"
-        message += "Possible causes:\n"
-        message += "- too wide tab so it does not hit the board,\n"
-        message += "- annotation is placed inside the board,\n"
-        message += "- ray length is not sufficient,\n"
-        raise RuntimeError(message) from None
+        raise TabError(origin, direction, [
+            "too wide tab so it does not hit the board",
+            "annotation is placed inside the board",
+            "ray length is not sufficient"
+        ])
 
     def _makeTabFillet(self, tab: Polygon, tabFace: LineString, fillet: KiLength) \
             -> Tuple[Polygon, LineString]:
@@ -933,7 +942,7 @@ class Substrate:
         Add fillets to inner corners which will be produced by a mill with
         given radius.
         """
-        EPS = 1 # This number is intentionally near KiCAD's resolution of 1nm to not enclose narrow slots, but to preserve radius
+        EPS = 1000 # This number is intentionally near KiCAD's resolution of 1nm to not enclose narrow slots, but to preserve radius
         RES = 32
         if millRadius < EPS:
             return
